#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['tuxtrigger']

package_data = \
{'': ['*']}

install_requires = \
['tuxsuite', 'squad-client', 'jinja2', 'requests', 'PyYAML']

entry_points = \
{'console_scripts': ['tuxtrigger = tuxtrigger.__main__:main']}

setup(name='tuxtrigger',
      version='0.6.5',
      description='TuxTrigger Project by Linaro',
      author='Pawel Szymaszek',
      author_email='pawel.szymaszek@linaro.org',
      url='https://tuxsuite.com/',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
